@@ -28,7 +28,6 @@ from .random_variables import (
 from .distance import (
     Distance,
     NoDistance,
-    IdentityFakeDistance,
     AcceptAllDistance,
     SimpleFunctionDistance,
     PNormDistance,
@@ -72,6 +71,9 @@ from .sampler import (
     MappingSampler,
     DaskDistributedSampler,
     RedisEvalParallelSampler,
+    RedisStaticSampler,
+    RedisEvalParallelSamplerServerStarter,
+    RedisStaticSamplerServerStarter,
     MulticoreEvalParallelSampler,
     ConcurrentFutureSampler)
 from .inference import ABCSMC
