@@ -13,11 +13,14 @@ import pyabc
                         ])
 def sampler(request):
     s = request.param()
-    yield s
     try:
-        s.cleanup()
-    except AttributeError:
-        pass
+        yield s
+    finally:
+        # release all resources
+        try:
+            s.shutdown()
+        except AttributeError:
+            pass
 
 
 def test_basic(sampler: pyabc.sampler.Sampler):
